struct HDF5ChunkNoCompressionSerializer: public HDF5ChunkSerializer {
    int get_buffer_size(const struct SeedeeNDArray* array, const void* compression_opts, int n_opts)
    {
        if (compression_opts != NULL) {
            return INVALID_COMPRESSION_OPTS;
        }
        int size = array->itemsize * n_elements(array);
        assert(array->size == size);
        return size;
    }

    int serialize_ndarray(
        const struct SeedeeNDArray* array, const void* compression_opts, int n_opts,
        char* buf, int buf_len, char** meta)
    {
        if (compression_opts != NULL) {
            return INVALID_COMPRESSION_OPTS;
        }
        // TODO: zero_copy?
        // TODO: skip check?
        int buffer_size = get_buffer_size(array, compression_opts, n_opts);
        if (buf_len < buffer_size)
        {
            return BUFFER_TOO_SMALL;
        }

        // Serialize array
        std::memcpy(buf, array->data, array->size);

        // Create metadata
        json j = basic_metadata();
        j["data_format"] = FORMAT_HDF5_CHUNK;
        j["compression"] = COMPRESSION_NONE;
        j["compression_opts"] = nullptr;
        json array_json;
        int err;
        err = array_to_json(array, array_json);
        if (err != 0) {
            return err;
        }
        j["array"] = array_json;

        json_dump_c_str(j, meta);

        return array->size;
    }

    int get_data_size(
        const json& j, const char* buf, int buf_len, bool* zero_copy, struct SeedeeNDArray* array)
    {
        if (j.at("compression_opts") != nullptr)
        {
            return INVALID_COMPRESSION_OPTS;
        }
        auto j_array = j.at("array");
        int ret = json_to_array(j_array, array);

        // TODO: zero_copy
        *zero_copy = false;

        return ret;
    }

    int deserialize_ndarray(
        const json& j, const char* buf, int buf_len,
        bool zero_copy, struct SeedeeNDArray* array)
    {
        // TODO: zero_copy?
        // TODO: skip check?
        if (buf_len < array->size)
        {
            return BUFFER_TOO_SMALL;
        }

        // Deserialize array
        std::memcpy(array->data, buf, array->size);

        std::vector<int> shape = j.at("array").at("shape");
        assert(shape.size() == array->ndims);
        memcpy(array->shape, shape.data(), array->ndims * sizeof(*(array->shape)));

        return array->size;
    }
};
