const char *seedee_errstr(int err) {
   switch (err) {
        case INVALID_PROTOCOL: return "Invalid protocol: must be seedee";
        case INVALID_MAJOR_VERSION: return "Invalid major version: must be 1";
        case INVALID_MINOR_VERSION: return "Invalid minor version: must be 0";
        case INVALID_FORMAT: return "Invalid format: must be hdf5_chunk";
        case INVALID_COMPRESSION: return "Invalid compression type: must be one of 0, 1, 32004, or 32008";
        case INVALID_COMPRESSION_OPTS: return "Invalid compression opts: none default compression opts not supported";
        case BUFFER_TOO_SMALL: return "Buffer too small: use get_buffer_size/get_data_size to get minimum size";
        case COMPRESSION_FAILED: return "Compression or decompression failed: compression library reported failure";
        case INVALID_NUMBER_OF_BLOCKS: return "Invalid number of blocks: must be 1";
        case INVALID_BLOCK_SIZE: return "Invalid block size: cannot be negative";
        case INVALID_STRIDE: return "Invalid stride: the array has an unsupported memory layout";
        case INVALID_ARRAY: return "Invalid array: given size, shape, itemsize, and/or stride are inconsistent";
        case INVALID_JSON: return "Invalid JSON: the given metadata is not a valid JSON string";
        case INCOMPLETE_METADATA: return "Incomplete metadata: at least one expected key is missing in JSON string";
    };
    return "Invalid error value";
}


std::vector<int> get_shape(const struct SeedeeNDArray* array)
{
    std::vector<int> v;
    for (int i = 0; i < array->ndims; ++i)
    {
        v.push_back(array->shape[i]);
    }

    return v;
}


int n_elements(const struct SeedeeNDArray* array)
{
    // TODO: empty arrays?
    int n = 1;
    for (int i=0; i<array->ndims; ++i) {
        n *= array->shape[i];
    }
    return n;
}


int n_elements(const std::vector<int>& shape)
{
    // TODO: empty arrays?
    int n = 1;
    for (auto e:shape) {
        n *= e;
    }
    return n;
}


int array_to_json(const SeedeeNDArray* array, json& j) {
    if (array->stride != nullptr) {
        return INVALID_STRIDE;
    }

    j["size"] = array->size;
    j["ndims"] = array->ndims;
    j["shape"] = get_shape(array);
    j["stride"] = nullptr;
    j["datatype"] = array->datatype;
    j["itemsize"] = array->itemsize;
    j["byteorder"] = array->byteorder;
    return 0;
}


int json_to_array(const json& j_array, SeedeeNDArray* array) {
    array->size = j_array.at("size");
    array->ndims = j_array.at("ndims");
    array->itemsize = j_array.at("itemsize");
    array->datatype = j_array.at("datatype").get<char>();
    array->byteorder = j_array.at("byteorder").get<char>();

    if (j_array.at("stride") != nullptr) {
        return INVALID_STRIDE;
    }

    return array->size;
}


int json_dump_c_str(const json& j, char** c_str) {
    std::string s = j.dump();
     // TODO: Is there a better way?
    *c_str = (char*)malloc(s.size() + 1);
    strcpy(*c_str, s.c_str());
    return s.size();
}


// read big endian integer from buffer
template<typename T>
T read_be(const void* buf) {
    const unsigned char* data = (const unsigned char*) buf;
    T num = 0;
    int shift = 8*(sizeof(T) - 1);
    for (int i = 0; i < sizeof(T); i++) {
        num |= (T)data[i] << shift;
        shift -= 8;
    }
    return num;
}


// write big endian integer to buffer
template<typename T>
void write_be(void* buf, T num) {
    unsigned char* data = (unsigned char*) buf;
    int shift = 8*(sizeof(T) - 1);
    for (int i = 0; i < sizeof(T); i++) {
        data[i] = num >> shift;
        shift -= 8;
    }
}
