#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "seedee/seedee.h"

#define DIM0            6
#define DIM1            8

int main() {
    char* meta;
    struct SeedeeNDArray warray, rarray;
    char* buf;
    int status;
    bool zero_copy;

    int shape[2] = {DIM0, DIM1};

    short wdata[DIM0][DIM1];

    printf("seedee version: %s\n", seedee_version);

    /*
     * Initialize data
     */
    int counter = 0;
    for (int i=0; i<DIM0; i++) {
        for (int j=0; j<DIM1; j++) {
            wdata[i][j] = counter;
            ++counter;
        }
    }

    /*
     * Print the data to the screen.
     */
    printf ("Original Data:\n");
    for (int i=0; i<DIM0; i++) {
        printf (" [");
        for (int j=0; j<DIM1; j++)
            printf (" %3d", wdata[i][j]);
        printf ("]\n");
    }

    warray.data = (char*)wdata;
    warray.size = DIM0*DIM1*sizeof(**wdata);
    warray.ndims = 2;
    warray.shape = (int *)&shape;
    warray.stride = NULL;
    warray.byteorder = '<';
    warray.itemsize = sizeof(**wdata);
    warray.datatype = 'i';

    printf("ndims = %i\n", warray.ndims);
    printf("shape = %i, %i\n", warray.shape[0], warray.shape[1]);
    printf("itemsize = %i\n", warray.itemsize);
    printf("datatype = %c\n", warray.datatype);
    printf("byteorder = %c\n", warray.byteorder);

    int max_buf_len = seedee_get_buffer_size(
        &warray, FORMAT_HDF5_CHUNK, COMPRESSION_NONE, NULL, 0);

    printf("max_buf_len = %i\n", max_buf_len);

    buf = malloc(max_buf_len);

    // Test error handling
    int buf_len = seedee_serialize_ndarray(
        &warray, 42, COMPRESSION_NONE, NULL, 0, buf, max_buf_len, &meta);

    assert(buf_len == INVALID_FORMAT);

    printf("%s\n", seedee_errstr(buf_len));

    // Test compression
    buf_len = seedee_serialize_ndarray(
        &warray, FORMAT_HDF5_CHUNK, COMPRESSION_NONE, NULL, 0, buf, max_buf_len, &meta);

    printf("buf_len = %i\n", buf_len);

    rarray.size = seedee_get_data_size(meta, buf, buf_len, &zero_copy, &rarray);

    printf("data_size = %i\n", rarray.size);
    rarray.data = malloc(rarray.size);
    rarray.shape = malloc(rarray.ndims*sizeof(int));

    status = seedee_deserialize_ndarray(meta, buf, buf_len, false, &rarray);
    if (status < 0) {
        printf("Error serializing data\n");
        return -1;
    }

    printf("ndims = %i\n", rarray.ndims);
    printf("shape = %i, %i\n", rarray.shape[0], rarray.shape[1]);
    printf("itemsize = %i\n", rarray.itemsize);
    printf("datatype = %c\n", rarray.datatype);
    printf("byteorder = %c\n", rarray.byteorder);

    printf ("Deserialized Data:\n");
    for (int i=0; i<DIM0; i++) {
        printf (" [");
        for (int j=0; j<DIM1; j++)
            printf (" %3d", ((short*)(rarray.data))[i*DIM1 + j]);
        printf ("]\n");
    }

    int size = 4000*4000*sizeof(long long int);
    char * data = malloc(size);
    char * datacopy = malloc(size);
    memcpy(data, datacopy, size);
    struct SeedeeNDArray array;
    array.data = data;
    array.size = size;
    array.ndims = 2;
    array.shape = malloc(2*sizeof(int));
    array.shape[0] = 4000;
    array.shape[1] = 4000;
    array.stride = NULL;
    array.byteorder = '<';
    array.itemsize = 2;
    array.datatype = 'i';

    max_buf_len = seedee_get_buffer_size(
        &array, FORMAT_HDF5_CHUNK, COMPRESSION_BITSHUFFLE, NULL, 0);

    buf = realloc(buf, max_buf_len);

    buf_len = seedee_serialize_ndarray(
    &array, FORMAT_HDF5_CHUNK, COMPRESSION_BITSHUFFLE, NULL, 0, buf, max_buf_len, &meta);

    printf("buf_len = %i\n", buf_len);
    assert(buf_len < array.size);

    array.size = seedee_get_data_size(meta, buf, buf_len, &zero_copy, &array);

    seedee_deserialize_ndarray(meta, buf, buf_len, false, &array);

    for (size_t i=0; i< 4000*4000*sizeof(int); ++i) {
        if (i < 10) {printf("%lu\n", i);}
        assert(datacopy[i] == array.data[i]);
    }

    printf ("Done:\n");
    return 0;
}