#include "seedee/seedee.h"
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

int main()
{
   printf("seedee version: %s\n", seedee_version);
   printf("Setup\n");
   const int ndims = 2;
   const int w = 3;
   const int h = 4;
   int shape[ndims] = {w, h};
   double a[w][h] = {
      {0, 1, 2, 3},
      {4, 5, 6, 7},
      {8, 9, 10, 11}
   };

   SeedeeNDArray array;
   array.data = (char*)a;
   int nelems = 1;
   for (int i = 0; i < ndims; ++i) {
      nelems *= shape[i];
   }
   array.size = nelems*sizeof(double);
   array.ndims = ndims;
   array.shape = shape;
   array.stride = NULL;
   array.itemsize = sizeof(double);
   array.datatype = 'f';
   array.byteorder = '<';

   char* meta;

   printf("Serialize data\n");

   int buf_len = seedee_get_buffer_size(
      &array, FORMAT_HDF5_CHUNK, COMPRESSION_BITSHUFFLE, NULL, 0);

   printf("buf_len = %i\n", buf_len);

   char* buf = (char*)malloc(buf_len);

   // Test error handling
   int nbytes = seedee_serialize_ndarray(
      &array, FORMAT_INVALID, COMPRESSION_NONE, NULL, 0, buf, buf_len, &meta);

   assert(nbytes == INVALID_FORMAT);

   printf("%s\n", seedee_errstr(nbytes));

   // Test compression
   nbytes = seedee_serialize_ndarray(
      &array, FORMAT_HDF5_CHUNK, COMPRESSION_BITSHUFFLE, NULL, 0, buf, buf_len, &meta);

   printf("nbytes = %i\n", nbytes);
   printf("meta = %s\n", meta);

   printf("Deserialize data\n");
   SeedeeNDArray out_array;
   bool zero_copy;
   int size = seedee_get_data_size(meta, buf, nbytes, &zero_copy, &out_array);

   printf("size = %i\n", size);

   out_array.data = (char*)malloc(size);
   out_array.shape = (int*)malloc(out_array.ndims*sizeof(int));

   nbytes = seedee_deserialize_ndarray(meta, buf, nbytes, zero_copy, &out_array);

   printf("nbytes = %i\n", nbytes);

    printf ("Deserialized Data:\n");
    for (int i=0; i<w; i++) {
        printf (" [");
        for (int j=0; j<h; j++)
            printf (" %g", ((double*)(out_array.data))[i*h + j]);
        printf ("]\n");
    }

   return 0;
}
