set -uex

source versions.txt

# JSON
new_json_version=3.11.3
if [[ ${new_json_version} != ${json_version} ]]; then
    # Can always be restored vie `git checkout json`
    rm -rf json/nlohmann_json
    mkdir json/nlohmann_json
    curl -L https://github.com/nlohmann/json/releases/download/v${new_json_version}/json.tar.xz | tar -Jxf - -C json/nlohmann_json --strip-components=1
    if [ $? -eq 0 ]; then
        json_version=${new_json_version}
    else
        echo "ERROR dowloading nlohmann json"
        exit 1
    fi
fi


# LZ4
new_lz4_version=1.9.4
if [[ ${new_lz4_version} != ${lz4_version} ]]; then
    # Can always be restored vie `git checkout lz4`
    rm -rf lz4/lz4
    mkdir -p lz4/lz4
    curl -L https://github.com/lz4/lz4/archive/refs/tags/v${new_lz4_version}.tar.gz | tar -zxf - -C lz4/lz4 --strip-components=1
    if [ $? -eq 0 ]; then
        lz4_version=${new_lz4_version}
    else
        echo "ERROR dowloading LZ4"
        exit 1
    fi
fi


# zlib
new_zlib_version=1.3.1
if [[ ${new_zlib_version} != ${zlib_version} ]]; then
    # Can always be restored vie `git checkout zlib`
    rm -rf zlib/zlib
    mkdir -p zlib/zlib
    curl -L https://github.com/madler/zlib/archive/refs/tags/v${new_zlib_version}.tar.gz | tar -zxf - -C zlib/zlib --strip-components=1
    if [ $? -eq 0 ]; then
        zlib_version=${new_zlib_version}
    else
        echo "ERROR dowloading zlib"
        exit 1
    fi
fi



echo "json_version=${json_version}" > versions.txt
echo "lz4_version=${lz4_version}" >> versions.txt
echo "zlib_version=${zlib_version}" >> versions.txt